/*
 *	loque\FieldObjekt.java
 *
 *	gesehene Spieler
 *
 *  $Id: FieldObjekt.java,v 1.10 2002/07/13 20:09:31 richard Exp $
 *
 */
package loque;

import java.io.Serializable;

// log4j Klassen importieren
import org.apache.log4j.Category;
import org.apache.log4j.Priority;
import org.apache.log4j.NDC;

/**
 * Datenstruktur fr gesehene Spieler
 */
public class FieldObjekt implements java.io.Serializable {

	static Category cat = Category.getInstance( FieldObjekt.class.getName());
	private static final int MAX_AGE = 12;

  /** absolute Koordinaten */
  public int x, y;
  /** absolute Geschw. */
  public int vx, vy, va;
  /** absolute Richtung */
  public int dir;
	/** Spielernummer, falls verfgbar */
	public short num;
	/** absoluter Winkel zu Objekt */
	public int angle;
	/** Entfernung zum Spieler */
	public int dist;
	/** wie oft bereits eine Extrapolation vorgenommen wurde */
	public int age;
	/** extrapolierte Position */
	public int nextX, nextY;
	/** Teamzugehoerigkeit */
	public short team;
	public static final short Ball = 0;
	public static final short Friend = 1;
	public static final short Enemy = 2;
	public static final short Unknown = 3;
	
  /**
   * Konstruktor
   *
   */
  public FieldObjekt( int iX, int iY, int iVx, int iVy, int iVa, int iDir, 
											short sNum, int iAngle, int iDist, short sTeam ){
	  x=iX; y=iY; vx=iVx; vy=iVy; va=iVa; dir=iDir; num = sNum; angle = iAngle; 
		dist = iDist; team = sTeam;	nextX = x+vx; nextY = y+vy;
  }
	/**
	 * extrapoliert die Position fuer neuen Zyklus (keine Distanz/Winkel!!!)
	 * @param cycles Zahl der Zyklen, die seit letzem Mal verstrichen sind
	 * @return true, wenn Objekt noch halbwegs gueltig
	 */
	public boolean extrapolate( int cycles ){
		int decay = team == 0 ? 94 : 100;
		for( int a = 0; a < cycles; a++ ){
			x += vx; y += vy; vx = decay*vx/100; vy = decay*vy/100; va = decay*va/100;
		}
		age += cycles; nextX = x+vx; nextY= y+vy;
		return age <= MAX_AGE;
	}

	/**
	 * extrapoliert die Distanz und den Winkel fuer neuen Zyklus
	 * @param px neue Spielerposition X
	 * @param py neue Spielerposition Y
	 * @return this
	 */
	public FieldObjekt completeExtrapolation( int px, int py ){
		double dx = x - px, dy = py - y;
		dist = (int)Math.sqrt( dx*dx+dy*dy );
		angle = (int)(18000.0/Math.PI*Math.atan2( dx, dy ));
		return this;
	}

}

/*
 * Version History:
 *
 * $Log: FieldObjekt.java,v $
 * Revision 1.10  2002/07/13 20:09:31  richard
 * trainingslager ver1
 *
 * Revision 1.9  2002/07/07 14:31:03  richard
 * Diverses privatisiert
 *
 * Revision 1.8  2002/07/05 16:40:07  richard
 * allgemeines Optimieren
 *
 * Revision 1.7  2002/07/01 22:28:02  richard
 * aktuelle KI
 *
 * Revision 1.6  2002/06/29 18:10:12  richard
 * KI ueberarbeitet
 *
 * Revision 1.5  2002/06/23 21:06:47  richard
 * History in WorldModell eingefuegt, IO_Translate korrigiert bezueglich goal,
 * KI-Daten an DataCore verbessert, ...
 *
 * Revision 1.4  2002/06/02 22:29:33  richard
 * habe diverse Korrekturen vorgenommen, KI geschrieben,
 * einige IO Bugs entfernt, usw.
 *
 * Revision 1.3  2002/06/02 13:17:45  richard
 * Log4J aktiviert, Header korrigiert, Einheitenkommentare hinzugefuegt, Umrechnung der Einheiten hinzugefuegt, allgemeine Korrekturen
 *
 * Revision 1.2  2002/05/28 14:31:52  bbleiffus
 * log4j Grundstruktur auskommentiert eingefgt
 *
 * Revision 1.1  2002/05/28 12:33:06  richard
 * moving files
 *
 * Revision 1.4  2002/05/25 16:50:07  richard
 * rudimentaeres StartUp erstellt, Haupt-Playerschleife und Login hinzugefuegt
 *
 * Revision 1.3  2002/05/25 14:55:23  richard
 * added package-statements, worked on the WorldModell, added Line-Class
 *
 * Revision 1.2  2002/05/24 14:21:20  richard
 * habe Log korrigiert
 *
 *
 */
