/*
 *	loque\IO_Translate.JAVA
 *
 *  kodiert und dekodiert Nachrichten
 *
 *  $Id: IO_Translate.java,v 1.27 2002/07/14 14:36:04 richard Exp $
 *
 */
package loque;

import java.util.StringTokenizer;


// log4j Klassen importieren
import org.apache.log4j.Category;
import org.apache.log4j.Priority;
import org.apache.log4j.NDC;

/**
 * kodiert und dekodiert Nachrichten
 */
public class IO_Translate {
  /** Referenz auf Player */
  private Player parent;

  //** Rckgabe-Konstanten */
	public static final int TimeOver = -4;
  public static final int Goal  = -3;
  public static final int HalfTime = -2;
  public static final int Error = -1;

  /** letzte FeelMessage */
  private int lastFeelTime;

  //log4j
  static Category cat = Category.getInstance( IO_Translate.class.getName());

  /**
   * Konstruktor
   * @param Referenz auf Player
   */
  public IO_Translate( Player parent ){
    this.parent = parent;
    lastFeelTime=0;
  }

  /**
   * Prft ob die empfangene Message vollstndig ist.
   * Dies ist nur ein grober Test indem die Anzahl der offenen Klammern
   * mit den der schliesenden Klammern verglichen wird.
   *
   * Die Methode wird normal nur bei einer exception aufgerufen!
   *
   * @param Message vom RoboCup Server
   */
  private boolean check_Message(String Message){
    int Klammercounter=0;

    for (int i=0; i < Message.length(); i++){
      if ( Message.charAt(i) == '(') Klammercounter++;
      if ( Message.charAt(i) == ')') Klammercounter--;
    }
    if (Klammercounter == 0) return true;
    return false;
  }


  /**
   * Mit dieser Methode finden wir heraus ob eine "see Message" vom
   * rcss Server "leer" ist. Bsp.  (see 12000)
   * Ebenfals nur ein grober Test
   *
   * @param Message vom RoboCup Server
   */
  private boolean check_see_Message_empty(String Message){
    if (Message.length() < 12) // Maximale realistische Zeit mit 5 Stellen bercksichtigt
      if (Message.startsWith("(see ")) // handelt es sich um eine see Nachricht
        if (check_Message(Message)) //Ist die Message einigermasen in ord.
          return true;

    return false;
  }

   /**
    * Zerleget eine (see ... ) Message und leitet die
    * den Objekttype und die Daten weiter an send_see_Message();
    *
    * @param Message vom RoboCup Server
    *
    */
  private void decode_see_Message(String Message){

    int time=0;

    /* Vor der inneren While schleife ist nicht klar wie viel Informationen (data)
     * eines gesehen Objekts ankommen.
     * Nach der inneren While schleife wird durch die
     * Zhlvariable i klar wie viele es wirklich sind und die Daten werden
     * in ein passendes array int data[] verschoben und weitergeleitet.
     * Informationen von denen wir nichts wissen werden auf null gesetzt.
     */
    int data[] = {0,0,0,0,0,0};

    //Zhlvariable
    int i=0;

    String type = "";

    /*  Zerlegt Message in einzelne Tokens und als Trennzeichen werden die "()"
     *  benutzt. Bsp.:
     *  Message vom RoboServer: (see 152 ((f t l) 15 14) ((f b c) 1 15 13 16 1))
     *  tokenizer.nextToken()="see 152 "
     *  tokenizer.nextToken()="f t l"
     *  tokenizer.nextToken()=" 15 14"
     *  tokenizer.nextToken()=" "
     *  tokenizer.nextToken()="f b c"
     *  tokenizer.nextToken()=" 1 15 13 16 1"
     *  keine weiteren Tokens.
     */
    StringTokenizer tokenizer = new StringTokenizer(Message, "()");

    /*
     *  extract Time
     *  nextToken() = "see time "
     */
    time = Integer.parseInt( tokenizer.nextToken().substring(4).trim() );

    // bereitet World auf neue See Messages vor.
    parent.world.resetSee( time );


    while (tokenizer.hasMoreTokens()) {
      // Objekttyp z.b "f c"
      type = tokenizer.nextToken();


      /* ObjektDaten werden getrennt
       * Bsp. tokenizer.nextToken()= " 37.3 -33"
       * Diese Daten werden in der nchsten while schleife weiter getrennt
       * Die Anzahl der Daten wird in i festgehalten.
       */

      //Zhlvariable
      i=0;

      try{
        StringTokenizer dataTokenizer = new StringTokenizer(tokenizer.nextToken());
        while (dataTokenizer.hasMoreTokens()){
          data[i] = Math.round(100*Float.valueOf(dataTokenizer.nextToken()).floatValue());
          i++;
        }
      }catch (NumberFormatException ex){
        cat.error("NumberFormatException bei der Bestimmun der Objektdaten in Message:" +Message);
        cat.error("in type: " + type);
        cat.error("in data["+i+"]: " + data[i]);
        cat.error("Message is " + (check_Message(Message) ? "OK" : "INCORREKT") + Message);
        return;
      }catch (java.util.NoSuchElementException ex) {
        /*
         * Im obigen Try Block kommt es zu einer exception wenn eine "see Message
         * keine Objekte enthlt also z.B (see 5600).
         * Da dieser Fall nur sehr selten auftritt wird er mit einer exception
         * abgefangen.
         * !! Es erfolgt dann keine Fehlermeldung. !!
         */
        if (!check_see_Message_empty(Message)){
          cat.error("NoSuchElementException bei der Bestimmun der Objektdaten in Message:" +Message);
          cat.error("in type: " + type);
          cat.error("in data["+i+"]: " + data[i]);
          cat.error("Message is " + (check_Message(Message) ? "OK" : "INCORREKT") + Message);
        }
        return;
    	}

      //weiterleiten zu send_see_Message
      send_see_Message(type,data);

      //eliminiert einen berflssigen Token = " "
      try{
        tokenizer.nextToken();
      }
      catch(java.util.NoSuchElementException ex){
        cat.error("NoSuchElementException beim eliminieren eines berflssigen Token:" +Message);
        cat.error("in type: " + type);
        cat.error("in data["+i+"]: " + data[i]);
        cat.error("Message is " + (check_Message(Message) ? "OK" : "INCORREKT") + Message);
        return;
      }
    }// end While
  }


  /**
   * Zerleget eine (sense_body ... ) Message und sendet
   * ein Feel Objekt an World Modell
   *
   * Wenn die Zeitangabe in der Message < als der Spiele cycle
   * wird die Nachricht nicht decodiert und der Spiele cycle wir
   * zurckgeliefert
   *
   * @param   Message vom rcssserver
   */
  private void decode_body_Message(String Message){
    int time=0;
    // Zerlegen der Messages in die "einzelen Nachrichten".
    StringTokenizer tokenizer = new StringTokenizer(Message,"()");
    StringTokenizer dataTokenizer;

    // z.B "stamina" oder "view_mode"
    String type;

    short viewmode=0;
    short viewwidth=0;
    int stamina=0;
    int speed=0;
    int speedDir=0;
    int headDir=0;

    try{
    //Auslesen der Zeit
    time = Integer.parseInt( tokenizer.nextToken().substring(11).trim() );
    }catch (java.util.NoSuchElementException ex) {
      cat.error("NoSuchElementException bei der Bestimmung der Zeit in Message: " +Message);
      cat.error("Message is " + (check_Message(Message) ? "OK" : "INCORREKT") + Message);
      return;
    }catch (NumberFormatException ex){
      cat.error("NumberFormatException bei der Bestimmung der Zeit in Message: " +Message);
      cat.error("Message is " + (check_Message(Message) ? "OK" : "INCORREKT") + Message);
      return;
    }

    /*
     * Wenn Time <= lastFeelTime wird die Message nicht decodiert und Cycle (lastFeelTime)wird
     * zurckgegeben.
     */
    if (time > lastFeelTime){
      lastFeelTime = time;

      // bereitet World auf die Aufnahme neuer Daten vor.
      parent.world.resetFeel();


      while (tokenizer.hasMoreTokens()) {
        // Bsp. tokenizer.nextToken() = "view_mode high normal"
        // nextToken() wird weiter zerlegt
        dataTokenizer = new StringTokenizer(tokenizer.nextToken());

        try{
          // Bsp. view_mode
          type = dataTokenizer.nextToken();
        }catch (java.util.NoSuchElementException ex){
	        cat.error("NoSuchElementException bei der Bestimmung des Types in Message:" +Message);
          cat.error("Message is " + (check_Message(Message) ? "OK" : "INCORREKT") + Message);
          return;
	      }

        try{
          // Hier sehehn wir nach um was fr einen Type es sich handelt und
          // bestimmen hierfr gleich die Daten,
          if (type.compareTo("stamina")==0)
            stamina = Math.round(100*Float.valueOf(dataTokenizer.nextToken()).floatValue());

          if (type.compareTo("speed")==0){
            speed = Math.round(100*Float.valueOf(dataTokenizer.nextToken()).floatValue());
            speedDir = Math.round(100*Float.valueOf(dataTokenizer.nextToken()).floatValue());
          }

          if (type.compareTo("head_angle")==0)
            headDir = Math.round(100*Float.valueOf(dataTokenizer.nextToken()).floatValue());


          if (type.compareTo("view_mode")==0){

            String buffer = dataTokenizer.nextToken();
            if (buffer.compareTo("high")==0)
              viewmode=1;
            else
              viewmode=0;

            buffer = dataTokenizer.nextToken();
            if (buffer.compareTo("narrow")==0)
              viewwidth=3;
            if (buffer.compareTo("normal")==0)
              viewwidth=2;
            if (buffer.compareTo("wide")==0)
              viewwidth=1;
          }
        // Da gab es einen Fehler, wir brechen aber noch nicht ab, vieleicht ist ja noch was
        // brauchbares dabei.
        }catch (NumberFormatException ex){
          cat.error("NumberFormatException bei der Bestimmun der Objektdaten in Message:" +Message);
          cat.error("in type: " + type);
          cat.error("Message is " + (check_Message(Message) ? "OK" : "INCORREKT") + Message);
        }catch (java.util.NoSuchElementException ex) {
	        cat.error("NoSuchElementException bei der Bestimmun der Objektdaten in Message:" +Message);
	        cat.error("in type: " + type);
          cat.error("Message is " + (check_Message(Message) ? "OK" : "INCORREKT") + Message);
        }
        try{
          //beseitigt einen berflssign Token
          tokenizer.nextToken();
        }catch (java.util.NoSuchElementException ex) {
	        cat.error("NoSuchElementException bei der Beseitigung eines berflssigen tokens in Message:" +Message);
          cat.error("Message is " + (check_Message(Message) ? "OK" : "INCORREKT") + Message);
	    }//end while



      }
      // bergabe der Daten an World als Feel Objekt
      parent.world.receive(new Feel(viewmode,viewwidth,stamina,speed,speedDir,headDir));
    }//end if time

  }


  /**
   * Dekodieren und weitergeben.
   * Die eigentliche decodierung erfolgt z.T. in private Methoden um den Code
   * einigermaen bersichtlich zu halten.
   *
   * @param   Message vom rcssserver
   * @return  Spielcycle
   *
   */
  public int decode( String Message ){

    /* Welchen Typ von Message haben wir?
     * In "decode_see_Message" werden die einzelnen Objekt Typen und Daten
     * herausgefiltert und dann an send_see_Message bergeben.
     */
    if ( Message.startsWith("(see")){
      decode_see_Message(Message);
    }

    /* Die Zeitangabe in dieser Message bestimmt unseren Cycle.
     * Ist die Zeitangabe <= dem aktuellen Cycle wird die Message ignoriert.
     */
    if ( Message.startsWith("(sense_body")){
      decode_body_Message(Message);
    }

    // Wir hren was, nur was?
    if ( Message.startsWith( "(hear " )){
      StringTokenizer tokenizer = new StringTokenizer(Message);

      String hearMessageParts[] = new String[4];
      /*
       * Wenn alles richtig luft, dann steht in hearMessageParts[i] folgendes:
       * i = 0 : "(hear"
       *     1 : Zeitnangabe
       *     2 : referee oder Winkelangabe
       *     3 : Nachricht
       */
      try{
        for (int i = 0; i < 4; i++) hearMessageParts[i] = tokenizer.nextToken();
      }
      catch(java.util.NoSuchElementException ex){
        cat.error("Fehler beim decodieren einer hear Message" + ex + "Message: " + Message);
        return Error;
      }

      //berprfe ob die Message vom Referee kommt.
      if (hearMessageParts[2].startsWith("referee")) {
        // Nachricht kommt vom Referee!
        cat.debug("Nachricht vom referee: " + hearMessageParts[3]);

        // Halbzeit, gebe Ausnahme-Wert direkt zurck damit wir reagieren knnen
        if (hearMessageParts[3].startsWith("half_time")){
          cat.info("Message half_time erkannt");
          return HalfTime;
        }
        // Halbzeit, gebe Ausnahme-Wert direkt zurck damit wir reagieren knnen
        if (hearMessageParts[3].startsWith("time_over")){
          return TimeOver;
        }
         // Tor, gebe Ausnahme-Wert direkt zurck damit wir reagieren knnen
        if ((hearMessageParts[3].startsWith("goal_l"))|(hearMessageParts[3].startsWith("goal_r"))){
          cat.info("Message goal erkannt");
          return Goal;
        }
        // Freisto-, Absto-, Eckball-, rechts: Wird in den Datacore geschrieben, die Ki fragt das dann ab
        if (hearMessageParts[3].startsWith("free_kick_r")
						|| hearMessageParts[3].startsWith("goal_kick_r")
						|| hearMessageParts[3].startsWith("corner_kick_r")
            || hearMessageParts[3].startsWith("kick_in_r")
            || hearMessageParts[3].startsWith("kick_off_r")){
          // entscheide ob fr oder gegen uns
					if ( parent.datacore.side == parent.datacore.RightSide ){
            parent.datacore.free_kick_friend = true; parent.datacore.free_kick_enemy = false;
					}else{
            parent.datacore.free_kick_enemy = true; parent.datacore.free_kick_friend = false;
					}
          return lastFeelTime;
        }
        // Freisto-, Absto-, Eckball- links: Wird in den Datacore geschrieben, die Ki fragt das dann ab
        if (hearMessageParts[3].startsWith("free_kick_l")
						|| hearMessageParts[3].startsWith("goal_kick_l")
						|| hearMessageParts[3].startsWith("corner_kick_l")
            || hearMessageParts[3].startsWith("kick_in_l")
 						|| hearMessageParts[3].startsWith("kick_off_l")){
          // entscheide ob fr oder gegen uns
					if ( parent.datacore.side == parent.datacore.LeftSide ){
            parent.datacore.free_kick_friend = true; parent.datacore.free_kick_enemy = false;
					}else{
            parent.datacore.free_kick_enemy = true; parent.datacore.free_kick_friend = false;
					}
          return lastFeelTime;
        }
        // rechter Tormann hat den Ball gefangen. Sag ich dem Datacore
        if (hearMessageParts[3].startsWith("goalie_catch_ball_l")){
          cat.info("Message goalie_catch_ball_l erkannt");

          // Welcher Tormann unserer oder der andere.
          if (parent.datacore.side == parent.datacore.LeftSide)
            parent.datacore.goalie_catch_ball_friend = true;
          else
            parent.datacore.goalie_catch_ball_enemy = true;

          return lastFeelTime;
        }
        // rechter Tormann hat den Ball gefangen. Sag ich dem Datacore
        if (hearMessageParts[3].startsWith("goalie_catch_ball_r")){
          cat.info("Message goalie_catch_ball_r erkannt");
          // Welcher Tormann unserer oder der andere.
          if (parent.datacore.side == parent.datacore.RightSide)
            parent.datacore.goalie_catch_ball_friend = true;
          else
            parent.datacore.goalie_catch_ball_enemy = true;

          return lastFeelTime;
        }


        // Das Spiel geht weiter, setze die Variablen zurck denn die gelten nicht mehr
        if (hearMessageParts[3].startsWith("play_on")){
          cat.info("play_on");
          parent.datacore.goalie_catch_ball_enemy = false;
          parent.datacore.goalie_catch_ball_friend = false;
          parent.datacore.free_kick_friend = false;
          parent.datacore.free_kick_enemy = false;

          return lastFeelTime;
        }
      }

      /*
       * Message kommt von einem Spieler
       * Der folgende Code wurde sollte es den Spielern ermglichen unter
       * einander zu kommunizieren. Dies sollte in einer spteren Entwicklung
       * einmal mglich werden, jedoch bis zu Abgabe konnte dieser Teil nicht
       * mehr realisiert werden (von KI-Seite her).
       */

/*
      else {
        cat.info("Message von einem anderen Spieler" + hearMessageParts[3]);
        //Message von einem unsere Spieler
        //Wenn die Nachricht mit unserem teamnamen beginnt ist sie hoffentlich fr uns
        if (hearMessageParts[3].startsWith("\"" + parent.datacore.teamname)){
          //zerlege die Message
          StringTokenizer hearTokenizer = new StringTokenizer(hearMessageParts[3]);

          // Teamname
          hearTokenizer.nextToken();
          // empfnger
          short empfaenger = Short.parseShort( hearTokenizer.nextToken() );
          //Wenn die Nachricht nicht fr uns ist dann breche das decoding ab
          if ( empfaenger ==  parent.datacore.num){
            short sender = Short.parseShort( hearTokenizer.nextToken());
            int messageNumber = Integer.parseInt( hearTokenizer.nextToken());
            String befehl = hearTokenizer.nextToken();

            Hear test = new Hear(empfaenger, sender, messageNumber, befehl);

          // Methode ext. noch nicht.
          //parent.ki.receive(test);

          }
          return lastFeelTime;
        }
      }//end if/else referee vs Spieler

*/

    }//end hear

    /*
     * Das die Reaktion auf ein init Kommando wird hier bearbeitet
     * Hier finden wir heraus ob das Anmelden funktioniert hat, auf welcher Seite
     * wir spielen und welche Spielernummer wir haben. Die letzten beiden werden
     * direkt im Datacore eingetragen. Als Rckgabewert bekommen wir entweder 0 fr
     * alles in Ordnung (Wird als Cycle interpret.) bzw. Error wenn etwas nicht
     * funktioniert.
     */
    if( Message.startsWith( "(init " ) ){
      try{
        parent.datacore.side = Message.charAt(6) == 'l' ? DataCore.LeftSide : DataCore.RightSide;
        parent.datacore.num = (short)Integer.parseInt( Message.substring(8,10).trim() );
	      return 0;
      }catch( IndexOutOfBoundsException dummy ){
        cat.error("Fehler bei der Anmeldung am Server: " +dummy);
        cat.error("Message: " +Message);
        return Error;
      }
    }

    /*
     * Die Error Message soll auch ab und an mal vorkommen und hier wird diese kurz
     * erkannt und ein Error als Rckgabewert geliefert.
     */
    if ( Message.equals( "" ) || Message.startsWith("(error")){
      return Error;
    }


    return lastFeelTime;
  }



  /**
   * Kodieren und versenden
   * Die bergenenen Objekte mssen eine Methode toString besitzen,
   * welche dir Objete in korrekte rcssserver Befehle konvertiert.
   *
   * @param Kommando-Objekt an RoboCup Server
   */
  public void encode( Object Message ){
    parent.network.send(Message.toString());
  }

  /**
   * bekommt die Daten von decode und erzeugt daraus die richtigen Objekte.
   * Diese werden dann an world.receive weitergereicht
   */
  private void send_see_Message(String Type, int Dataint[]){
    //System.out.println("|"+Type+"|"+Data+"|");  //debug

    //debug
    //System.out.println(Dataint[0]+"|"+Dataint[1]+"|"+Dataint[2]+"|"+Dataint[3]+"|"+Dataint[4]+"|"+Dataint[5]);

    /* hier wird der Datentype bestimmt und in das entsprechende Objekt eingetragen
     * Type ball, line ,flag
     */


    //Ball
    if( (Type.startsWith("b")) | ( Type.startsWith("B")) ){
      //See( short iTeam, short iNum, int iDist, int iDir, int iVrel, int iVdir )
      //Ich hoffe das stimmt ???
      parent.world.receive(new See( See.Ball,(short)0,Dataint[0],Dataint[1],Dataint[2],Dataint[3]));
    }

    //Type == line
    if (Type.startsWith("l")){

      if (Type.startsWith("l l")){
        //Line(short Type, int Dist, int Dir)
        parent.world.receive(new Line((short)0,Dataint[0],Dataint[1]));
        return;
      }

      if (Type.startsWith("l t")){
        parent.world.receive(new Line((short)1,Dataint[0],Dataint[1]));
        return;
      }

      if (Type.startsWith("l r")){
        parent.world.receive(new Line((short)2,Dataint[0],Dataint[1]));
        return;
      }

      if (Type.startsWith("l b")){
        parent.world.receive(new Line((short)3,Dataint[0],Dataint[1]));
        return;
      }
    }



    //Tore
    if (Type.startsWith("g l")){
      parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5250 , 0 ));
      return;
    }
    if (Type.startsWith("g r")){
      parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5250 , 0 ));
      return;
    }


    //Type == flag
    if (Type.startsWith("f")){
      //Strafrume (Flagen)
      if (Type.startsWith("f p l t")){
        //Flag( int iDist, int iDir, int iAx, int iAy )
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , -3600 , -2018 ));
        return;
      }
      if (Type.startsWith("f p l c")){
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , -3600 , 0 ));
        return;
      }
      if (Type.startsWith("f p l b")){
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , -3600 , 2018 ));
        return;
      }
      if (Type.startsWith("f p r t")){
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , 3600 , -2018 ));
        return;
      }
      if (Type.startsWith("f p r c")){
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , 3600 , 0 ));
        return;
      }
      if (Type.startsWith("f p r b")){
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , 3600 , 2018 ));
        return;
      }

      //Mittelline (Flagen)
      //Es gibt 3 Flaggen die mit c anfangen, Problem das die center Flag nur aus c "besteht"
      if (Type.startsWith("f c")){
        if (Type.startsWith("f c t")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 0 , -3400 ));
          return;
        }
        if (Type.startsWith("f c b")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 0 , 3400 ));
          return;
        }
        // Flag c
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , 0 , 0 ));
        return;
      }




      //Torpfosten  ??? Torgre ???
      if (Type.startsWith("f g l t")){
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5250 , -701 ));
        return;
      }
      if (Type.startsWith("f g l b")){
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5250 , 701 ));
        return;
      }
      if (Type.startsWith("f g r t")){
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5250 , -701 ));
        return;
      }
      if (Type.startsWith("f g r b")){
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5250 , 701 ));
        return;
      }



      //Banden links mit Eckfahnen
      if (Type.startsWith("f l")){


        if (Type.startsWith("f l t")){
          //Banden Flagen links oben
          if (Type.startsWith("f l t 30")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5750 , -3000 ));
            return;
          }
          if (Type.startsWith("f l t 20")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5750 , -2000 ));
            return;
          }
          if (Type.startsWith("f l t 10")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5750 , -1000 ));
            return;
          }
          //Eckfahne links oben
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5250  , -3400 ));
          return;
        }

        if (Type.startsWith("f l b")){
          //Banden Flagen links unten
          if (Type.startsWith("f l b 30")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5750 , 3000 ));
            return;
          }
          if (Type.startsWith("f l b 20")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5750 , 2000 ));
            return;
          }
          if (Type.startsWith("f l b 10")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5750 , 1000 ));
            return;
          }
          //Eckfahne links unten
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5250  , 3400 ));
          return;
        }
        //Banden Flagge links mitte
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5250  , 0 ));
        return;
      }


      //Banden rechts mit Eckfahnen
      if (Type.startsWith("f r")){

        if (Type.startsWith("f r t")){
          //Banden Flagen rechts oben
          if (Type.startsWith("f r t 30")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5750 , -3000 ));
            return;
          }
          if (Type.startsWith("f r t 20")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5750 , -2000 ));
            return;
          }
          if (Type.startsWith("f r t 10")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5750 , -1000 ));
            return;
          }
          //Eckfahne rechts oben
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5250  , -3400 ));
          return;
        }

        if (Type.startsWith("f r b")){
          //Banden Flagen rechts unten
          if (Type.startsWith("f r b 30")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5750 , 3000 ));
            return;
          }
          if (Type.startsWith("f r b 20")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5750 , 2000 ));
            return;
          }
          if (Type.startsWith("f r b 10")){
            parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5750 , 1000 ));
            return;
          }
          //Eckfahne rechts unten
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5250  , 3400 ));
          return;
        }
        //Banden Flagge rechts mitte
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5750  , 0));
        return;
      }


      //Banden oben
      if (Type.startsWith("f t")){
        //Banden oben links
        if (Type.startsWith("f t l 10")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -1000 , -3900 ));
          return;
        }
        if (Type.startsWith("f t l 20")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -2000 , -3900 ));
          return;
        }
        if (Type.startsWith("f t l 30")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -3000 , -3900 ));
          return;
        }
        if (Type.startsWith("f t l 40")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -4000 , -3900 ));
          return;
        }
        if (Type.startsWith("f t l 50")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5000 , -3900 ));
          return;
        }

        //Banden oben rechts
        if (Type.startsWith("f t r 10")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 1000 , -3900 ));
          return;
        }
        if (Type.startsWith("f t r 20")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 2000 , -3900 ));
          return;
        }
        if (Type.startsWith("f t r 30")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 3000 , -3900 ));
          return;
        }
        if (Type.startsWith("f t r 40")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 4000 , -3900 ));
          return;
        }
        if (Type.startsWith("f t r 50")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5000 , -3900 ));
          return;
        }
	      //Bande oben mitte
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , 0 , -3900 ));
        return;
      }


      //Banden unten
      if (Type.startsWith("f b")){

        //Banden unten links
        if (Type.startsWith("f b l 10")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -1000 , 3900 ));
          return;
        }
        if (Type.startsWith("f b l 20")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -2000 , 3900 ));
          return;
        }
        if (Type.startsWith("f b l 30")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -3000 , 3900 ));
          return;
        }
        if (Type.startsWith("f b l 40")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -4000 , 3900 ));
          return;
        }
        if (Type.startsWith("f b l 50")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , -5000 , 3900 ));
          return;
        }

        //Banden unten rechts
        if (Type.startsWith("f b r 10")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 1000 , 3900));
          return;
        }
        if (Type.startsWith("f b r 20")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 2000 , 3900 ));
          return;
        }
        if (Type.startsWith("f b r 30")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 3000 , 3900 ));
          return;
        }
        if (Type.startsWith("f b r 40")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 4000 , 3900 ));
          return;
        }
        if (Type.startsWith("f b r 50")){
          parent.world.receive(new Flag( Dataint[0], Dataint[1] , 5000 , 3900 ));
          return;
        }
        //Bande unten mitte
        parent.world.receive(new Flag( Dataint[0], Dataint[1] , 0 , 3900 ));
        return;
      }
    }

    //Spieler
    if (Type.startsWith("p") || Type.startsWith("P") ){
      StringTokenizer tokenizer = new StringTokenizer(Type);
      //See( short iTeam, short iNum, int iDist, int iDir, int iVrel, int iVdir ){
      switch (tokenizer.countTokens()){
        // Spieler mit unbekannter Mannschaft
        case 1:{
          parent.world.receive( new See( See.Unknown , (short) 0 , Dataint[0], Dataint[1] , Dataint[2], Dataint[3]) );
          break;
        }
        // Spieler mit Mannschaft aber ohne Nummer
        case 2:{
          // liefert p zurck
          tokenizer.nextToken();
          //nextToken liefert nun den teamnamen in "
          if (tokenizer.nextToken().compareTo( "\"" + parent.datacore.teamname + "\"" ) == 0 )
            parent.world.receive( new See( See.Friend , (short) 0 , Dataint[0], Dataint[1] , Dataint[2], Dataint[3]) );
          else
            parent.world.receive( new See( See.Enemy , (short) 0 , Dataint[0], Dataint[1] , Dataint[2], Dataint[3]) );
          break;
        }
        // Spieler mit Mannschaft und Nummer, 3 und 4 werden gleich behandelt,
        // 4 ist nur der goalie
        case 3:
        case 4:{

          // liefert p zurck
          tokenizer.nextToken();
          try{
            //nextToken liefert nun den teamnamen in "
            if (tokenizer.nextToken().compareTo( "\"" + parent.datacore.teamname + "\"" ) == 0 ){
              //nextToken liefert nun die Spielernummer
              parent.world.receive( new See( See.Friend , Short.parseShort( tokenizer.nextToken().trim() ) , Dataint[0], Dataint[1] , Dataint[2], Dataint[3]) );
            }
            else{
              //nextToken liefert nun die Spielernummer
              parent.world.receive( new See( See.Enemy , Short.parseShort( tokenizer.nextToken().trim() ) , Dataint[0], Dataint[1] , Dataint[2], Dataint[3]) );
            }
          }
          catch(NumberFormatException ex){
            cat.error("Fehler bei der Umwandlung der Spielernummer von String to Integer");
          }

          break;
        }
        // Da ist was schief gelaufen
        default :{
        cat.error("unbekannter Spielertype: " + Type );
        }
      }
    }
  }//end send_see_Message
}//end IO_Translate



/*
 * Version History:
 *
 * $Log: IO_Translate.java,v $
 * Revision 1.27  2002/07/14 14:36:04  richard
 * kosmetische Aenderungen
 *
 * Revision 1.26  2002/07/14 12:02:31  richard
 * Trainigslager Ver.3
 *
 * Revision 1.25  2002/07/13 20:09:31  richard
 * trainingslager ver1
 *
 * Revision 1.24  2002/07/07 20:23:15  matthias
 * Fehler im Kommentar korrigiert
 *
 * Revision 1.23  2002/07/06 20:02:47  matthias
 * Noch mehr Kommentare
 *
 * Revision 1.19  2002/07/02 18:58:06  richard
 * auf neuen Algorithmus umgestellt
 *
 * Revision 1.1  2002/07/02 12:03:28  richard
 * neuer Algorithmus fuer WorldModell
 *
 * Revision 1.18  2002/06/29 17:57:33  matthias
 * wir "fuehlen" jetzt den Ball   "((B) etc.)"
 *
 * Revision 1.16  2002/06/24 22:00:56  richard
 * KI bearbeitet, intercept auskommentiert, da noch nicht zufriedenstellend...
 *
 * Revision 1.15  2002/06/24 18:53:46  matthias
 * Goal, nochmaliger debug.
 * play_on eingefuegt
 *
 * Revision 1.14  2002/06/23 21:06:47  richard
 * History in WorldModell eingefuegt, IO_Translate korrigiert bezueglich goal,
 * KI-Daten an DataCore verbessert, ...
 *
 * Revision 1.13  2002/06/20 18:47:51  matthias
 * Hear Klasse erstellt.
 * IO_Trans und Say klar zur Kommunikation
 * !!! NOCH NICHT GETESTET !!!
 *
 * Revision 1.12  2002/06/18 10:15:26  richard
 * geschwindigkeiten und richtungen werden jetzt berechnet
 *
 * Revision 1.11  2002/06/14 11:53:30  richard
 * IO_Trans gedebugt
 *
 * Revision 1.10  2002/06/14 08:46:27  matthias
 * IO_Translate komplett berarbeitet.
 *
 * Revision 1.9  2002/06/04 15:50:46  richard
 * KI optimiert
 *
 * Revision 1.8  2002/06/04 11:10:49  matthias
 * seitenwechsel eingefuegt
 *
 * Revision 1.7  2002/06/03 22:15:27  richard
 * KI erweitert und korrigiert, ChangeView erstellt und eingebaut,
 * laeuft jetzt
 *
 * Revision 1.6  2002/06/02 22:29:33  richard
 * habe diverse Korrekturen vorgenommen, KI geschrieben,
 * einige IO Bugs entfernt, usw.
 *
 * Revision 1.5  2002/06/02 13:17:45  richard
 * Log4J aktiviert, Header korrigiert, Einheitenkommentare hinzugefuegt, Umrechnung der Einheiten hinzugefuegt, allgemeine Korrekturen
 *
 * Revision 1.4  2002/06/02 08:37:18  matthias
 * IO_Translate: encode fertiggestellt
 * Turn und Turn_Neck hinzugefgt
 * Turn, Turn_Neck, Catch, Dash, Kick, Move, Say Methode toString hinzugefgt
 *
 * Revision 1.3  2002/06/01 20:59:48  matthias
 * sense_body messages werden erkannt und weitergeleitet, nicht getestet.
 * private set_body_Message hinzugefgt
 * decode zerlegt jetzt auch hear messages
 * private send_see_Message etwas optimiert.
 *
 * Revision 1.2  2002/05/28 14:31:52  bbleiffus
 * log4j Grundstruktur auskommentiert eingefgt
 *
 * Revision 1.1  2002/05/28 12:33:06  richard
 * moving files
 *
 * Revision 1.7  2002/05/27 20:13:39  matthias
 * IOTrans: Ball Erkennung hinzugefgt
 * See: Koordinaten korrigiert 1/100
 *
 * Revision 1.6  2002/05/26 15:29:06  matthias
 * habe "decode" und  "send_see_Message" implementiert  (siehe Status der einz. Methoden).
 * decode wurde getestet. send_see_Message" nicht vollstndig.
 *
 * Revision 1.5  2002/05/25 19:12:18  richard
 * corrected Log
 *
 *
 */