/*
 *	loque\MonitorClient.java
 *
 *	sendet DataCore an Monitor
 *
 *  $Id: MonitorClient.java,v 1.17 2002/07/07 14:31:03 richard Exp $
 *
 */

package loque;

import java.io.*;
import java.net.*;

// log4j Klassen importieren
import org.apache.log4j.Category;
import org.apache.log4j.Priority;
import org.apache.log4j.NDC;

/**
 * MonitorClient
 *
 * Die Klasse sendet ber eine TCP Verbindung den DataCore zum Monitor.
 *
 * @author <a href="mailto:bbleifus@rumms.uni-mannheim.de">Bernd Bleifu</a>
 */

 public class MonitorClient {
  /** @see Player */
  private Player parent;
  static Category cat = Category.getInstance( MonitorClient.class.getName());
  /** socket fr das zu sendende DataCore Objekt */
  private Socket sock;
  // Variable fr Ausgabestream festlegen
  private ObjectOutputStream out;

  /**
   * Erstellt eine neue <code>MonitorClient</code> Instanz.
   *
   * @param rParent Angabe zu welchem Spieler die MonitorClient Instanz gehrt
   * @param adress Die Adresse des MonitorServer durch seinen Namen oder seine IP Adresse
   * @param port Der Port des MonitorServer
   */
  public MonitorClient( Player rParent, String adress, int port )throws IOException{
    parent=rParent;
    try {
      // Erstellen eines neuen Sockets und binden an den MonitorServer
      sock = new Socket( adress, port );
      // Einen Ausgabestream zum versenden des DataCore Objekts vobereiten
      out = new ObjectOutputStream(sock.getOutputStream());
    } catch (UnknownHostException uhe) {
      cat.error(adress + " konnte nicht gefunden werden.");
      throw new IOException( "Monitor nicht gefunden !" );
    }
  }

  /**
   * Sendet ein DataCore <code>Objekt</code> zum MonitorServer
   */
  public void send() {
    try {
      // Das DataCore Objekt versenden
      out.writeObject((Object)parent.datacore);
      // Zum debuggen, welcher DataCore gesendet wurde
      cat.debug( "DataCore zum Cycle " + parent.datacore.cycle + " gesendet" );
      // Ausgabestream zurcksetzen, damit ein neues Objekt versendet werden kann
      out.reset();
    } catch( IOException ioe ) {
      // "Selbstmord" bei bertragungsfehler, damit nicht der ganze Thread steht
      parent.monClient = null;
      cat.error( "Verbindung getrennt" );
    }
  }
}

/*
 * Version History:
 *
 * $Log: MonitorClient.java,v $
 * Revision 1.17  2002/07/07 14:31:03  richard
 * Diverses privatisiert
 *
 * Revision 1.16  2002/07/03 12:51:29  bbleiffus
 * Dokumentation fr javadoc eingefgt
 *
 * Revision 1.15  2002/06/10 21:02:22  bbleiffus
 * nderung an den Debugmessages aller Netzwerkkomponenten.
 * Aufruf des MonitorClient gendert !
 *
 * Revision 1.14  2002/06/09 12:48:59  bbleiffus
 * Erste einwandfrei funktionierende Version von
 * MonitorClient und MonitorServer mit Testausgabe.
 * Noch kein kompletter Test der Daten gemacht !
 *
 * Revision 1.13  2002/06/09 08:24:43  bbleiffus
 * Testausgaben eingebaut um den versendeten Inhalt zu prfen
 *
 * Revision 1.12  2002/06/08 18:15:27  bbleiffus
 * Neue private Methode in MonitorServer zum besseren Debuggen
 * Struktur in MonitorClient verndert. Es wird nur noch ein Stream
 * geffnet und zum senden benutzt, nicht jedesmal ein Neuer.
 *
 * Revision 1.11  2002/06/07 14:07:35  bbleiffus
 * MonitorClient sendet den DataCore jetzt wie er soll
 *
 * Revision 1.10  2002/06/07 11:59:43  bbleiffus
 * Es wird ein Objekt (wahrscheinlich DataCore) vom MonitorClient
 * gesendet und vom MonitorServer empfangen.
 * TCP Verbindung steht, aber bisher nur eine bertragung nicht dauernd !
 *
 * Revision 1.9  2002/06/06 15:12:33  bbleiffus
 * nur Formatierung gendert
 *
 * Revision 1.8  2002/06/05 10:37:36  bbleiffus
 * socket erst nach bertragung schlieen
 * war vorher mist ihn gleich wieder zu schlieen
 *
 * Revision 1.7  2002/06/04 19:15:28  richard
 * Empfangsteil in Loque eingebaut
 *
 * Revision 1.6  2002/06/04 18:42:36  richard
 * MonitorClient entschaerft
 *
 * Revision 1.5  2002/06/04 18:19:06  bbleiffus
 * knappe Kommentare eingefgt
 *
 * Revision 1.4  2002/06/04 17:56:20  bbleiffus
 * Erste Version von Client und Server
 * NOCH NICHT GETESTET !!!
 *
 * Revision 1.3  2002/06/02 13:17:45  richard
 * Log4J aktiviert, Header korrigiert, Einheitenkommentare hinzugefuegt, Umrechnung der Einheiten hinzugefuegt, allgemeine Korrekturen
 *
 * Revision 1.2  2002/05/28 14:31:52  bbleiffus
 * log4j Grundstruktur auskommentiert eingefgt
 *
 * Revision 1.1  2002/05/28 12:33:06  richard
 * moving files
 *
 * Revision 1.3  2002/05/25 14:55:23  richard
 * added package-statements, worked on the WorldModell, added Line-Class
 *
 * Revision 1.2  2002/05/24 14:21:21  richard
 * habe Log korrigiert
 *
 *
 */
