/*
 *	loque\Netzwerk.java
 *
 *	sendet und empfaengt Daten vom Server
 *
 *  $Id: Netzwerk.java,v 1.13 2002/07/07 14:31:03 richard Exp $
 *
 */
package loque;

import java.io.*;
import java.net.*;

// log4j Klassen importieren
import org.apache.log4j.Category;
import org.apache.log4j.Priority;
import org.apache.log4j.NDC;

/**
 * Netzwerk
 *
 * Die Klasse verwaltet die UDP Verbindung vom/zum rcssserver.
 *
 * @author <a href="mailto:bbleifus@rumms.uni-mannheim.de">Bernd Bleifu</a>
 */
public class Netzwerk {
  /** @see Player */
  private Player parent;
  static Category cat = Category.getInstance( Netzwerk.class.getName());
  /** socket fr ankommende/abgehende Nachrichten */
  private DatagramSocket socket;
  /** IP Adresse des Servers */
  private InetAddress serverAddress;
  /** Portnummer des Servers */
  private int serverPort;

  /**
   * Erstellt eine neue <code>Netzwerk</code> Instanz.
   *
   * @param Parent Angabe zu welchem Spieler die Netzwerkinstanz gehrt
   * @param serverName Die Adresse des Servers durch seinen Namen oder seine IP Adresse
   * @param serverPort Der Port des Servers
   */
  public Netzwerk(Player Parent, String serverName, int serverPort) {
    parent = Parent;
      try {
        // erstellen des DatagramSocket und
        // binden an irgendeinen verfgbaren Port der lokalen Maschine
        this.socket        = new DatagramSocket();
        // Zieladresse setzen
        this.serverAddress = InetAddress.getByName(serverName);
        // Zielport setzen
        this.serverPort    = serverPort;
      } catch (SocketException se) {
        cat.error("Netzwerk: Konnte den Socket nicht erstellen.");
        System.exit(1);
      } catch (UnknownHostException uhe) {
        cat.error("Neztwerk: Konnte die IP Adresse des Servers nicht bestimmen.");
        System.exit(1);
      }
  }


  /**
   * Sendet einen <code>String</code> zum Server
   *
   * @param message ein <code>String</code> Wert, der die Nachricht enthlt,
   * welche zum Server gesendet werden soll.
   */
  public void send(String message) {
    try {
      // umwandeln der Nachricht in ein byte array
      byte[] buffer = message.getBytes();
      // Das Packet aus dem buffer erstellen und IP Addresse und Port des
      // Servers setzen
      DatagramPacket packet = new DatagramPacket(buffer, buffer.length, serverAddress, serverPort);
      // Das Packet senden
      socket.send(packet);
    } catch (IOException ioe) {
      cat.error("Netzwerk: Konnte das Paket nicht senden.");
      }
  }


  /**
   * Empfngt eine Nachricht vom Server
   *
   * @return einen <code>String</code> der die Nachricht vom Socket enthlt
   */
  public int receive() {
    // buffer fr die Server Nachricht anlegen
    byte[] buffer = new byte[4096];
    // DatagramPacket zum empfangen erstellen
    DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
    try {
      // Packet empfangen, die Methode wartet bis ein datagram ankommt
      socket.setSoTimeout(5000);
      socket.receive(packet);
    } catch (InterruptedIOException iioe) {
      return (-1);
    } catch (IOException ioe) {
      cat.error("Netzwerk: Konnte das Paket nicht empfangen.");
      }
      // Den Server Port fr folgende Aktionen bernehmen
      serverPort = packet.getPort();
      // Die Nachricht zur bersetzung weiterleiten
      String received = new String(packet.getData()).substring(0, packet.getLength())+" "; // cut off null characters in buffer
      // cat.info(received);
      return parent.ioTrans.decode(received);
    }
  }


/*
 * Version History:
 *
 * $Log: Netzwerk.java,v $
 * Revision 1.13  2002/07/07 14:31:03  richard
 * Diverses privatisiert
 *
 * Revision 1.12  2002/07/03 12:51:29  bbleiffus
 * Dokumentation fr javadoc eingefgt
 *
 * Revision 1.11  2002/06/29 21:24:40  bbleiffus
 * Timeout eingebaut, falls der Server nicht vorhanden ist
 * oder nicht antwortet
 *
 * Revision 1.10  2002/06/11 16:34:34  matthias
 * Problem mit der Buffergre korrigiert
 *
 * Revision 1.9  2002/06/10 21:02:22  bbleiffus
 * nderung an den Debugmessages aller Netzwerkkomponenten.
 * Aufruf des MonitorClient gendert !
 *
 * Revision 1.8  2002/06/06 15:12:33  bbleiffus
 * nur Formatierung gendert
 *
 * Revision 1.7  2002/06/04 17:57:18  bbleiffus
 * Keine wichtigen nderungen, nur kleine bersetzungen
 *
 * Revision 1.6  2002/06/03 22:15:27  richard
 * KI erweitert und korrigiert, ChangeView erstellt und eingebaut,
 * laeuft jetzt
 *
 * Revision 1.5  2002/06/03 21:07:25  bbleiffus
 * debug Ausgabe eingefgt
 *
 * Revision 1.4  2002/06/02 22:29:34  richard
 * habe diverse Korrekturen vorgenommen, KI geschrieben,
 * einige IO Bugs entfernt, usw.
 *
 * Revision 1.3  2002/06/02 13:17:45  richard
 * Log4J aktiviert, Header korrigiert, Einheitenkommentare hinzugefuegt, Umrechnung der Einheiten hinzugefuegt, allgemeine Korrekturen
 *
 * Revision 1.2  2002/05/28 14:31:52  bbleiffus
 * log4j Grundstruktur auskommentiert eingefgt
 *
 * Revision 1.1  2002/05/28 12:33:06  richard
 * moving files
 *
 * Revision 1.4  2002/05/25 15:06:05  richard
 * added PlayerReferenz to Netzwerk
 *
 * Revision 1.3  2002/05/24 14:21:21  richard
 * habe Log korrigiert
 *
 *
 */