/*
 *	loque\Player.java
 *
 *	ein Spieler
 *
 *  $Id: Player.java,v 1.17 2002/07/14 14:36:04 richard Exp $
 *
 */

package loque;

// log4j Klassen importieren
import org.apache.log4j.Category;
import org.apache.log4j.Priority;
import org.apache.log4j.NDC;

import java.io.IOException;

/**
 *	ein Spieler
 */
public class Player extends java.lang.Thread{

  static Category cat = Category.getInstance( Player.class.getName());

  /** MonitorClient falls Monitor verbunden */
  public volatile MonitorClient monClient;
	/** wird wahr, wenn beendet	werden soll */
	public volatile boolean suicide = false;
  public KI ki;
  public DataCore datacore;
  public WorldModell world;
  public IO_Translate ioTrans;
  public Netzwerk network;
  /**
   * Konstruktor
   */
  public Player( boolean goalie, String Serveradress, int Port, String Teamname )throws IOException{
	  monClient=null; ki = new KI(this); datacore = new DataCore();
	  world = new WorldModell(this); ioTrans = new IO_Translate(this);
	  network = new Netzwerk( this, Serveradress, Port );
		network.send("(init " + Teamname + " (version 7.0)" + (goalie?"(goalie))":")") );
		if( 0 != network.receive() )throw new IOException( "Login failed" );
		datacore.goalie = goalie;
		datacore.teamname = Teamname;
		ki.preStart();
  }
  /**
   * Start des Clients
   */
  public void run(){
		int lastCycle = 0, currCycle = 0;
		while(true){
			while( lastCycle >= currCycle ){
				try{
					currCycle = network.receive();
					if( suicide )break;
					if( currCycle == IO_Translate.HalfTime )
						ki.halfTime();
					if( currCycle == IO_Translate.Goal )
						ki.goal();
					if( currCycle == IO_Translate.TimeOver ){
						Loque.loop = false; suicide = true;
					}
				}catch( Throwable dummy ){
					cat.error( "unknow error during receive and decode:\n"+dummy );
				}
      }
			if( suicide )break;
			datacore.cycle = lastCycle = currCycle;
      try{
			  if( datacore.cycle > 0 && world.evaluate() ){
		  		ki.think();
	  		  if( monClient != null )monClient.send();
  			}
      }catch( Exception dummy){
				cat.error( "Exception in WorldModell.evaluate() or KI.think()!:\n"+
									 dummy );
			}
		}
		network.send( "(bye)" );
  }
}
/*
 * Version History:
 *
 * $Log: Player.java,v $
 * Revision 1.17  2002/07/14 14:36:04  richard
 * kosmetische Aenderungen
 *
 * Revision 1.16  2002/07/05 17:40:16  richard
 * shutdown vorbereitet
 *
 * Revision 1.15  2002/06/27 19:52:31  richard
 * debug-message entfernt
 *
 * Revision 1.14  2002/06/27 19:52:00  richard
 * bewegende Fields gefixt
 *
 * Revision 1.13  2002/06/23 21:06:47  richard
 * History in WorldModell eingefuegt, IO_Translate korrigiert bezueglich goal,
 * KI-Daten an DataCore verbessert, ...
 *
 * Revision 1.12  2002/06/18 15:16:39  richard
 * numFriends,... zu DataCore hinzugefuegt und in WorldModell eingebaut,
 * ListeningPort jetzt dynamisch ueber Commandozeile
 *
 * Revision 1.11  2002/06/18 11:47:10  richard
 * player crashsicherer
 *
 * Revision 1.10  2002/06/14 11:53:30  richard
 * IO_Trans gedebugt
 *
 * Revision 1.9  2002/06/13 14:50:01  matthias
 * Teamnamen eingefgt
 *
 * Revision 1.8  2002/06/04 15:50:46  richard
 * KI optimiert
 *
 * Revision 1.7  2002/06/04 12:17:03  richard
 * Halbzeit und ViewMode bearbeitet
 *
 * Revision 1.6  2002/06/04 11:10:49  matthias
 * seitenwechsel eingefuegt
 *
 * Revision 1.5  2002/06/03 22:15:27  richard
 * KI erweitert und korrigiert, ChangeView erstellt und eingebaut,
 * laeuft jetzt
 *
 * Revision 1.4  2002/06/02 22:29:34  richard
 * habe diverse Korrekturen vorgenommen, KI geschrieben,
 * einige IO Bugs entfernt, usw.
 *
 * Revision 1.3  2002/06/02 13:17:45  richard
 * Log4J aktiviert, Header korrigiert, Einheitenkommentare hinzugefuegt, Umrechnung der Einheiten hinzugefuegt, allgemeine Korrekturen
 *
 * Revision 1.2  2002/05/28 14:31:52  bbleiffus
 * log4j Grundstruktur auskommentiert eingefgt
 *
 * Revision 1.1  2002/05/28 12:33:06  richard
 * moving files
 *
 * Revision 1.5  2002/05/25 20:56:07  richard
 * Feel-Class hinzugefuegt and deren Auswertung
 *
 * Revision 1.4  2002/05/25 16:50:07  richard
 * rudimentaeres StartUp erstellt, Haupt-Playerschleife und Login hinzugefuegt
 *
 * Revision 1.3  2002/05/25 14:55:23  richard
 * added package-statements, worked on the WorldModell, added Line-Class
 *
 * Revision 1.2  2002/05/24 14:21:21  richard
 * habe Log korrigiert
 *
 *
 */
