/*
 *	loque\WorldModell.java
 *
 *	verarbeitet alle empfangenen Nachrichten
 *
 *  $Id: WorldModell.java,v 1.30 2002/07/14 12:02:31 richard Exp $
 *
 */

package loque;

// log4j Klassen importieren
import org.apache.log4j.Category;
import org.apache.log4j.Priority;
import org.apache.log4j.NDC;

/**
 *	verarbeitet alle empfangenen Nachrichten
 */
public class WorldModell {

	static Category cat = Category.getInstance( WorldModell.class.getName());

	/** Referenz auf Spieler */
	private Player parent;
	/** FlagNachrichten Cache */
	private Flag[] flagCache;
	/** Zahl der empfangenen Flag-Nachrichten */
	private short lastFlag;
	/** SeeNachrichten Cache */
	private See[] seeCache;
	/** Zahl der empfangenen See-Nachrichten */
	private short lastSee;
	/** gesehene Lines */
	private Line[] lineCache;
	/** Zahl der gesehene Lines */
	private short lastLine;
	/** feel */
	private Feel feelCache;

	private static final short MAX_FLAGS = 20;
	private static final short MAX_SEE = 22;
	/** zaehlt mit wieoft der Ball nicht gesehen wurde */
	private int scan_failed;
	/** besagt, ob ViewMode bereits geaendert wurde */
	private boolean scan_set;
	/** letzter evaluation-cycle */
	private int lastCycle;
	/** cycle-Nummer der See-Message */
  private int seeCycle;

	//============================================================================
	/**
	 * Konstruktor
	 */
	public WorldModell( Player rParent ){
		parent=rParent;
		flagCache = new Flag[MAX_FLAGS]; lastFlag = 0;
		seeCache = new See[MAX_SEE]; lastSee = 0;
		lineCache = new Line[4]; lastLine = 0;
		feelCache = null; scan_failed = 0;
		lastCycle = 0; seeCycle = -1;
	}

	//============================================================================
	/**
	 * setzt Werte zurueck, um neue See-Nachricht zu empfangen
	 */
	public void resetSee( int cycle ){
    seeCycle = cycle;
		for( int a = 0; a < lastSee; a++ )seeCache[a] = null; lastSee = 0;
		for( int a = 0; a < lastLine; a++ )lineCache[a] = null; lastLine = 0;
		for( int a = 0; a < lastFlag; a++ )flagCache[a] = null; lastFlag = 0;
	}
	//============================================================================
	/**
	 * setzt Werte zurueck, um neue Feel-Nachricht zu empfangen
	 */
	public void resetFeel(){
		feelCache = null;
	}
	//============================================================================
	/**
	 * empfaengt Nachricht und legt sie im Cache ab
	 */
	public void receive( Object Message ){
		if( Message instanceof Flag && lastFlag < MAX_FLAGS ){
			flagCache[lastFlag++] = (Flag)Message;
		}else if( Message instanceof See && lastSee < MAX_SEE ){
			seeCache[lastSee++] = (See)Message;
		}else if( Message instanceof Line && lastLine < 4 ){
			lineCache[lastLine++] = (Line)Message;
		}else if( Message instanceof Feel && feelCache == null ){
			feelCache = (Feel)Message;
		}
	}
	//============================================================================
	/**
	 * ViewMode wechseln
	 */
	private void changeScan( int wider ){
		if( scan_set )return;
		scan_failed+=wider;
		if( scan_failed == 3 || scan_failed < -9 ){
			if( scan_failed < -9 )scan_failed = 9;
			parent.datacore.viewWidth = ChangeView.Wide;
			parent.ioTrans.encode( new ChangeView( ChangeView.Wide, ChangeView.High ) );
		}
    if( scan_failed == -3 || scan_failed > 9 ){
			if( scan_failed > 9 )scan_failed = -9;
			parent.datacore.viewWidth = ChangeView.Narrow;
			parent.ioTrans.encode( new ChangeView( ChangeView.Narrow, ChangeView.High ) );
    }
    if( scan_failed == 0 ){
			parent.datacore.viewWidth = ChangeView.Normal;
			parent.ioTrans.encode( new ChangeView( ChangeView.Normal, ChangeView.High ) );
    }
		scan_set = true;
	}

	//============================================================================
	/**
	* auswerten
	* @return true, wenn ausgewertet Daten ausreichend
	*/
	public boolean evaluate(){
		DataCore data = parent.datacore;
    data.turnNeckRequest = 0; scan_set = false;
		if( data.teleported ){
			data.delete(); data.vx = 0; data.vy = 0; data.va = 0;
		}
    if( seeCycle == -1 ){ //Cycle ohne See-Message
  		if( feelCache == null ){
        cat.error( "no body-message in WorldModell.evaluate()" );
        return false;
      }
			if( data.dashpower > 0 ){
				data.vx += (int)(data.dashpower*0.6*Math.sin((data.dir)*Math.PI/18000.0));
				data.vy += -(int)(data.dashpower*0.6*Math.cos((data.dir)*Math.PI/18000.0));
			}
      data.x += data.vx; data.y += data.vy;
  		data.headdir = feelCache.headDir;
	  	data.stamina = feelCache.stamina/100;
      data.va = feelCache.speed;
 			data.speedDir = feelCache.speedDir;
  		feelCache	= null;
	    data.dir = DataCore.normAngle( data.dir+data.turnangle );
	  	data.vx = (int)(data.va*Math.sin((data.dir)*Math.PI/18000.0));
  		data.vy = -(int)(data.va*Math.cos((data.dir)*Math.PI/18000.0));
      extrapolate( data.cycle-lastCycle );
	  	data.turn_neckangle = 0; data.turnangle = 0; data.dashpower = 0;
      lastCycle = data.cycle;
			if( data.teleported )return false;
  		return true;
    }
    //Cycle mit See
		if( feelCache == null ){
      cat.error( "no body-message in WorldModell.evaluate()" );
      return false;
    }
    int dir = DataCore.normAngle( data.dir+data.headdir );
		// nach Aktion.Validate auf normale Sicht zurueckschalten
		if( data.viewWidth == ChangeView.Wide && scan_failed <= 0 )
			parent.ioTrans.encode( new ChangeView( ChangeView.Normal, ChangeView.High ) );
		data.seeCycle = true;
		// Extrapolation:
		boolean turned = false;
    if( !data.teleported && seeCycle < data.cycle ){
			int olddir = data.dir;
			dir = evalLines(); 
			if( DataCore.abs( DataCore.normAngle( olddir-data.dir ) ) >
					DataCore.abs( DataCore.normAngle( olddir-data.dir+
																						data.turn_neckangle+data.turnangle ) ) ){
				data.headdir = DataCore.normAngle( data.headdir+data.turn_neckangle );
				data.dir = DataCore.normAngle( data.dir-data.turn_neckangle );
				turned = true;
			}
			if( data.turnangle != 0 || data.turn_neckangle != 0 ){
				if( turned )olddir += data.turnangle;
				if( DataCore.abs( DataCore.normAngle( olddir-data.dir ) ) > 25 ){
					data.dir = olddir;
				}
			}
			evalFlags( dir ); evalObjects( dir );
    }
		if( data.dashpower > 0 ){
			data.vx += (int)(data.dashpower*0.6*Math.sin((data.dir)*Math.PI/18000.0));
			data.vy += -(int)(data.dashpower*0.6*Math.cos((data.dir)*Math.PI/18000.0));
		}
		data.x += data.vx; data.y += data.vy;
		// Sense_Body auswerten
		data.headdir = feelCache.headDir;
    if( !turned )data.dir = DataCore.normAngle( data.dir+data.turnangle );
		data.stamina = feelCache.stamina/100;
		data.va = feelCache.speed;
		data.speedDir = feelCache.speedDir;
		feelCache	= null;
		// Geschwindigkeit vektorisieren
		data.vx = (int)(data.va*Math.sin((data.dir)*Math.PI/18000.0));
		data.vy = -(int)(data.va*Math.cos((data.dir)*Math.PI/18000.0));
    extrapolate( data.cycle - lastCycle );
		// jetzt sind die Objekte dran...
    if( seeCycle == data.cycle ){
			System.out.println( "lateCycle" );
      dir = evalLines(); evalFlags( dir ); evalObjects( dir );
    }
		// Kopf zum ball drehen
		if( data.ball == null ){
			if( data.headdir < 8000 )data.turnNeckRequest = 4500;
			else  data.turnNeckRequest = -18000;
			changeScan( 1 );
		}
		else{
			if( data.ball.dist < 4000 && scan_failed < 0 ){
				changeScan( 1 );
			}
			if( DataCore.abs( DataCore.normAngle(  data.ball.angle-data.dir ) ) < 9100
          && scan_failed > 0 )
				changeScan( -1 );
			data.turnNeckRequest = DataCore.normAngle( data.ball.angle-data.dir-data.headdir );
		}
		// Aufraeumen
		data.turn_neckangle = 0; data.turnangle = 0; data.dashpower = 0;
    data.teleported = false; lastCycle = data.cycle; seeCycle = -1;
		return true;
	}
	//============================================================================
	/**
	 * berechnet Blickrichtung aus Liniendaten
	 */
	private int evalLines(){
		DataCore data = parent.datacore; int dir;
		if( lastLine == 0 ){
			if( data.headdir < 9000 )data.turnNeckRequest =	3000;
			else data.turnNeckRequest = -18000;
      changeScan( -1 );
			return DataCore.normAngle( data.dir+data.headdir );
		}
		else{
			short a;
			// Blickrichtung anhand jeder Linie einzel bestimmen!
			for( a = 0; a < lastLine; a++ )lineCache[a].calculate();
			// stimmen sie ungefaehr ueberein?
			boolean negative = false;
			for( a = 0; a < lastLine; a++ )
				if( lineCache[a].calcDiff( lineCache[0] ) > 9000 )negative=true;
			// wenn nicht muessen die naehesten invertiert werden
			if( negative ){
				// welche ist die Naeheste?
				Line nearest = lineCache[0];
				for( a = 1; a < lastLine; a++ )
					if( lineCache[a].dist < nearest.dist )nearest = lineCache[a];
				// welche invertieren?
				if( nearest.diffDir > 9000 ){
					for( a = 0; a < lastLine; a++ )
						if( lineCache[a].diffDir > 9000 )lineCache[a].invertDir();
				}else{
					for( a = 0; a < lastLine; a++ )
						if( lineCache[a].diffDir < 9000 )lineCache[a].invertDir();
				}
			}
			// jetzt sollten die Richtungen stimmen!
			// -> Mittelwert bilden
			dir = 0;
			for( a = 0; a < lastLine; a++){
				dir += lineCache[a].playerDir; lineCache[a] = null;
			}
			dir /= lastLine; lastLine = 0;
			data.dir = DataCore.normAngle( dir-data.headdir );
			return dir;
		}
	}

	//============================================================================
	/**
	 * berechnet Position aus Flagen
	 */
	private void evalFlags( int dir ){
  	DataCore data = parent.datacore;
		int x = data.x, y = data.y;
		if( lastFlag == 0 ){
			// nur Extrapolieren und Gegenmassnahmen treffen
			if( data.headdir < 9000 )data.turnNeckRequest = 9000;
			else data.turnNeckRequest = -18000;
			changeScan( 1 ); data.x = x; data.y = y; return;
		}
		int correctness=6-data.cycle+lastCycle;
		if( correctness < 1 )correctness = 1;
		int oldx=x,oldy=y;
		x*=correctness; y*=correctness;
		for( int a = 0; a < lastFlag; a++ ){
			correctness += flagCache[a].calcPosition(dir);
			x+=flagCache[a].px*flagCache[a].exact;
			y+=flagCache[a].py*flagCache[a].exact;
			flagCache[a]=null;
		}
		lastFlag = 0; x /= correctness; y /= correctness;
		data.x = x; data.y = y;
	}

	//============================================================================
	/**
	 * extrapoliert alte Objekte
	 */
  private void extrapolate( int cycles ){
	  FieldObjekt enemy[], friend[], unknown[];
		enemy = new FieldObjekt[11]; friend = new FieldObjekt[10];
    unknown = new FieldObjekt[21];
    DataCore data = parent.datacore;
		int a, numOld, px = data.x, py = data.y;
		// alte Daten retten und teilweise extrapolieren
		for( numOld = 0, a = 0; a < data.numEnemies; a++ )
			if( data.enemy[a].extrapolate( cycles ) )enemy[numOld++] = data.enemy[a];
    data.enemy = enemy; data.numEnemies = numOld;
    for( a = 0; a < numOld; a++ )data.enemy[a].completeExtrapolation( px, py );
		for( numOld = 0, a = 0; a < data.numFriends; a++ )
			if( data.friend[a].extrapolate( cycles ) )
        friend[numOld++] = data.friend[a];
    data.friend = friend; data.numFriends = numOld;
    for( a = 0; a < numOld; a++ )data.friend[a].completeExtrapolation( px, py );
		for( numOld = 0, a = 0; a < data.numUnknowns; a++ )
			if( data.unknown[a].extrapolate( cycles ) )
        unknown[numOld++] = data.unknown[a];
    data.unknown = unknown; data.numUnknowns = numOld;
    for( a = 0; a < numOld; a++ )
      data.unknown[a].completeExtrapolation( px, py );
		if( data.ball != null && data.ball.extrapolate( cycles ) )
      data.ball.completeExtrapolation( px, py );
    else data.ball = null;
  }

	//============================================================================
	/**
	 * erzeugt neue Objekte und mergt alte
	 */
	private void evalObjects( int dir ){
		// diverser benoetigter Scheiss:
		FieldObjekt old[], oldball = null, newobj[], newball = null;
		old = new FieldObjekt[21]; DataCore data = parent.datacore;
		int x = data.x, y = data.y, vx = data.vx, vy = data.vy;
		int a, numOld = 0;
		// alte Daten retten
		for( a = 0; a < data.numEnemies && numOld < 22; a++ )
      old[numOld++] = data.enemy[a];
		for( a = 0; a < data.numFriends && numOld < 22; a++ )
			old[numOld++] = data.friend[a];
		for( a = 0; a < data.numUnknowns && numOld < 22; a++ )
			old[numOld++] = data.unknown[a];
		oldball = data.ball;
		data.delete();
		// alte Daten, die sichbar sein sollten lschen->sichtbarer Halbwinkel?
		// erst Ball
		int visAngle = data.viewWidth == ChangeView.Narrow ? 2450 :
									( data.viewWidth == ChangeView.Normal ? 4700 : 9200 );
		if( oldball != null ){
			if( DataCore.abs( DataCore.normAngle( oldball.angle-dir ))
          < visAngle || oldball.dist < 300 )oldball=null;
			else newball = oldball;
		}
		// neue Daten erstellen
		newobj = new FieldObjekt[21];	int numNew = 0;
		for( a = 0; a < lastSee; a++ ){
			if( seeCache[a].team == See.Ball ){
        newball = seeCache[a].eval( x,y,vx,vy,dir );
				if( oldball != null && newball.dist > 1000 ){
					newball.vx = (70*newball.vx+30*oldball.vx)/100;
					newball.vy = (70*newball.vy+30*oldball.vy)/100;
					newball.va = (70*newball.va+30*oldball.va)/100;
					newball.dir = newball.va > 5 ? 
												(int)(18000.0/Math.PI*Math.atan2( 
															newball.vx,-newball.vy )) : 0;
					newball.nextX = newball.x+newball.vx;
					newball.nextY = newball.y+newball.vy;
				}
			}
      else if( numNew < 21 )
        newobj[numNew++] = seeCache[a].eval( x,y,vx,vy,dir );
			seeCache[a] = null;
		}
		lastSee = 0;
		// Daten mergen:
		// alte Daten uebernehmen 
		for( a = 0; a < numOld && numNew < 21; a++ ){
			if( old[a] != null ){
				// nur wenn nicht sichbar sein kann uebernehmen
				if( DataCore.abs( DataCore.normAngle( old[a].angle - dir ) )
            > visAngle ) newobj[numNew++] = old[a];
			}
		}
		// Daten in DataCore transferieren:
		data.ball = newball;
		short currUnknown = 0, currEnemy = 0, currFriend = 0;
		for( a = 0; a < numNew; a++ ){
			switch( newobj[a].team ){
				case See.Unknown:
          if( currUnknown < 21 )data.unknown[currUnknown++] = newobj[a]; break;
				case See.Friend:
          if( currFriend < 10 ) data.friend[currFriend++] = newobj[a]; break;
				case See.Enemy:
          if( currEnemy < 11 )data.enemy[currEnemy++] = newobj[a]; break;
			}
		}
		data.numFriends = currFriend; data.numEnemies = currEnemy;
    data.numUnknowns = currUnknown; seeCycle = -1;
	}
}

/*
 * Version History:
 *
 * $Log: WorldModell.java,v $
 * Revision 1.30  2002/07/14 12:02:31  richard
 * Trainigslager Ver.3
 *
 * Revision 1.29  2002/07/13 20:09:31  richard
 * trainingslager ver1
 *
 * Revision 1.28  2002/07/07 23:08:12  richard
 * verbesserte Richtungsextrapolation
 *
 * Revision 1.27  2002/07/07 22:35:33  richard
 * bessere Extrapolation der Geschwindigkeit
 *
 * Revision 1.26  2002/07/07 14:31:03  richard
 * Diverses privatisiert
 *
 * Revision 1.25  2002/07/07 14:05:50  richard
 * no message
 *
 * Revision 1.24  2002/07/06 11:31:49  richard
 * Kommentare eingefuegt
 *
 * Revision 1.23  2002/07/05 17:40:16  richard
 * shutdown vorbereitet
 *
 * Revision 1.22  2002/07/05 16:40:07  richard
 * allgemeines Optimieren
 *
 * Revision 1.21  2002/07/05 14:43:28  richard
 * intercept verbessert
 *
 * Revision 1.20  2002/07/05 11:10:07  richard
 * interception verbessert und Positionsbestimmung berarbeitet
 *
 * Revision 1.19  2002/07/04 19:43:00  richard
 * zum Ball laufen verbessert (hoffentlich)
 *
 * Revision 1.18  2002/07/02 18:58:06  richard
 * auf neuen Algorithmus umgestellt
 *
 * Revision 1.1  2002/07/02 12:03:28  richard
 * neuer Algorithmus fuer WorldModell
 *
 * Revision 1.17  2002/06/27 21:48:03  richard
 * KI gedebugt
 *
 * Revision 1.16  2002/06/27 14:24:55  richard
 * KI Restrukturierung hat begonnen
 *
 * Revision 1.15  2002/06/25 20:05:16  richard
 * intercept korrigiert und Weltmodell Historytracking verbessert
 *
 * Revision 1.14  2002/06/25 11:08:02  richard
 * intercept aktiviert, hat offensichtlich noch einen Bug, laeuft aber meistens
 *
 * Revision 1.13  2002/06/24 22:00:56  richard
 * KI bearbeitet, intercept auskommentiert, da noch nicht zufriedenstellend...
 *
 * Revision 1.12  2002/06/23 21:06:47  richard
 * History in WorldModell eingefuegt, IO_Translate korrigiert bezueglich goal,
 * KI-Daten an DataCore verbessert, ...
 *
 * Revision 1.11  2002/06/18 15:16:39  richard
 * numFriends,... zu DataCore hinzugefuegt und in WorldModell eingebaut,
 * ListeningPort jetzt dynamisch ueber Commandozeile
 *
 * Revision 1.10  2002/06/18 10:15:26  richard
 * geschwindigkeiten und richtungen werden jetzt berechnet
 *
 * Revision 1.9  2002/06/14 11:53:30  richard
 * IO_Trans gedebugt
 *
 * Revision 1.8  2002/06/13 14:08:39  richard
 * fehlertoleranz erhoeht
 *
 * Revision 1.7  2002/06/04 15:50:46  richard
 * KI optimiert
 *
 * Revision 1.6  2002/06/04 12:17:03  richard
 * Halbzeit und ViewMode bearbeitet
 *
 * Revision 1.5  2002/06/03 22:15:27  richard
 * KI erweitert und korrigiert, ChangeView erstellt und eingebaut,
 * laeuft jetzt
 *
 * Revision 1.4  2002/06/02 22:29:34  richard
 * habe diverse Korrekturen vorgenommen, KI geschrieben,
 * einige IO Bugs entfernt, usw.
 *
 * Revision 1.3  2002/06/02 13:17:45  richard
 * Log4J aktiviert, Header korrigiert, Einheitenkommentare hinzugefuegt, Umrechnung der Einheiten hinzugefuegt, allgemeine Korrekturen
 *
 * Revision 1.2  2002/05/28 14:31:52  bbleiffus
 * log4j Grundstruktur auskommentiert eingefgt
 *
 * Revision 1.1  2002/05/28 12:33:06  richard
 * moving files
 *
 * Revision 1.5  2002/05/25 20:56:07  richard
 * Feel-Class hinzugefuegt and deren Auswertung
 *
 * Revision 1.4  2002/05/25 16:50:07  richard
 * rudimentaeres StartUp erstellt, Haupt-Playerschleife und Login hinzugefuegt
 *
 * Revision 1.3  2002/05/25 14:55:23  richard
 * added package-statements, worked on the WorldModell, added Line-Class
 *
 * Revision 1.2  2002/05/24 14:21:21  richard
 * habe Log korrigiert
 *
 *
 */
