/*
 *	loque\MonitorServer.java
 *
 *	empfngt den DataCore am Monitor
 *
 *  $Id: MonitorServer.java,v 1.19 2002/07/06 10:29:02 richard Exp $
 *
 */

package monitor;

import java.io.*;
import java.net.*;
import loque.DataCore;

// log4j Klassen importieren
import org.apache.log4j.Category;
import org.apache.log4j.Priority;
import org.apache.log4j.NDC;

/**
 * MonitorServer
 *
 * Die Klasse erstellt eine TCP Verbindung zum MonitorClient und
 * stellt den empfangenen DataCore dem Monitor zur Verfgung.
 *
 * @author <a href="mailto:bbleifus@rumms.uni-mannheim.de">Bernd Bleifu</a>
 */
public class MonitorServer extends java.lang.Thread {
  static Category cat = Category.getInstance( MonitorServer.class.getName());
  /** socket fr ankommende Objekte */
  private ServerSocket server;
  // socket fr die kurze UDP Verbindung
  private Socket sock;
  // Variable des Eingabestreams fr das Objekt
  private ObjectInputStream in;

  /**
   * Erstellt eine neue <code>MonitorServer</code> Instanz.
   *
   * @param client Die Adresse des Client durch seinen Namen oder seine IP Adresse
   * @param port Der Port des kleinen UDP-Netzwerks in der Hauptklasse Loque
   * @param spieler Die Nummer des Spielers der vom Monitor angezeigt werden soll
   */
  public MonitorServer( String client, int port, int spieler ) {
    try {
      // erstellen eines ServerSockets fr die TCP-Verbindung und
      // binden an irgendeinen freien Port
      server = new ServerSocket(0);
      /* Erstellen einer kurzen UDP Verbindung zur Hauptklasse Loque um
         darber den MonitorClient zu starten (wodurch eine einfache
         Indentifizierung des Speilers mglich ist).
         bergeben wird die Spielernummer, sowie Adresse und Port auf
         denen dieser MonitorServer luft.*/
      DatagramSocket socket = new DatagramSocket();
      InetAddress clientadr = InetAddress.getByName(client);
      String localadr = InetAddress.getLocalHost().getHostAddress();
      int localport = server.getLocalPort();
      String aufruf = ( "connect" + " " + spieler + " " + localadr + " " + localport );
      // Ausgabe der Verbindungsnachricht zum debuggen
      cat.debug(aufruf);
      byte[] buffer = aufruf.getBytes();
      DatagramPacket packet = new DatagramPacket( buffer, buffer.length, clientadr, port );
      socket.send(packet);
      socket.close();
      // Auf Verbindung mit einem MonitorClient warten
      cat.debug( "[Server wartet auf Port " + localport + "]" );
      sock = server.accept();
      cat.debug( "[Verbindung zu " + sock.getInetAddress() + ":" + sock.getPort() + "]" );
      // Einen Eingabestream zum Empfangen des DataCore Objektes vorbereiten
      in = new ObjectInputStream(sock.getInputStream());
    } catch (UnknownHostException uhe) {
      cat.error( "Unbekannter Host" );
    } catch (SocketException se) {
      cat.error( "Socket konnte nicht erzeugt werden" );
    } catch (IOException io) {
      cat.error( "IO Fehler" );
    }
  }

  /**
   * Empfngt ein DataCore <code>Objekt</code> vom MonitorClient
   */
  public void run() {
    try {
      // Den empfangenen, nichtleeren DataCore dem Monitor zur Verfgung stellen
      while ((Monitor.data = (loque.DataCore)in.readObject()) != null) {
        // Zum debuggen, ob der richtige DataCore angekommen ist
        cat.debug( "Datacore zum Cycle " + Monitor.data.cycle + " empfangen" );
      }
    } catch (ClassNotFoundException cnfe) {
      cat.error( "Klasse nicht gefunden !" );
    } catch (IOException ioe) {
      cat.error( "bertragungsfehler !" );
    }
    cat.debug( "Verbindung beendet" );
  }
	
	/**
	 * Sendet einen ShutDown-Befehl an den Client...
	 * @return 1 if successful, 0 otherwise
	 */
	public static int shutdown( String Host, int Port ){
		try{
			DatagramSocket socket = new DatagramSocket();
			InetAddress clientadr = InetAddress.getByName( Host );
			String aufruf = ( "shutdown" );
			byte[] buffer = aufruf.getBytes();
			DatagramPacket packet = new DatagramPacket( buffer, buffer.length, clientadr, Port );
			socket.send(packet);
			socket.close();
		}
		catch( Exception dummy ){
			cat.info( "unable to send ShutDown-Command: "+dummy );
			return 0;
		}
		return 1;
	}
}

/*
 * Version History:
 *
 * $Log: MonitorServer.java,v $
 * Revision 1.19  2002/07/06 10:29:02  richard
 * shutdown in Monitor eingefgt
 *
 * Revision 1.18  2002/07/03 13:02:10  bbleiffus
 * Kleiner Tippfehler: Hatte ein int zum String gemacht
 *
 * Revision 1.17  2002/07/03 12:52:05  bbleiffus
 * Dokumentation fr javadoc eingefgt
 *
 * Revision 1.16  2002/06/18 15:16:39  richard
 * numFriends,... zu DataCore hinzugefuegt und in WorldModell eingebaut,
 * ListeningPort jetzt dynamisch ueber Commandozeile
 *
 * Revision 1.15  2002/06/10 21:03:12  bbleiffus
 * nderung an den Debugmessages aller Netzwerkkomponenten.
 * Aufruf des MonitorClient gendert !
 *
 * Revision 1.14  2002/06/10 11:12:19  richard
 * tcp-verbindung korrigiert
 *
 * Revision 1.13  2002/06/09 12:51:03  bbleiffus
 * Erste einwandfrei funktionierende Version von
 * MonitorClient und MonitorServer mit Testausgabe.
 * Noch kein kompletter Test der Daten gemacht !
 *
 * Revision 1.12  2002/06/09 08:50:04  bbleiffus
 * MonitorServer luft jetzt als Thread
 *
 * Revision 1.11  2002/06/09 08:24:06  bbleiffus
 * Testausgaben eingebaut um den versendeten Inhalt zu prfen
 *
 * Revision 1.10  2002/06/08 18:14:48  bbleiffus
 * Neue private Methode in MonitorServer zum besseren Debuggen
 * Struktur in MonitorClient verndert. Es wird nur noch ein Stream
 * geffnet und zum senden benutzt, nicht jedesmal ein Neuer.
 *
 * Revision 1.9  2002/06/08 10:59:39  bbleiffus
 * Typumwandlung des empfangenen Objektes in DataCore
 * Immernoch nur einmal empfangen ?!?
 *
 * Revision 1.8  2002/06/07 14:09:15  bbleiffus
 * kleine Modifikationen an der Variablenstruktur
 * Luft immernoch nicht wie er soll. Empfngt nur einmal !
 *
 * Revision 1.7  2002/06/07 11:59:06  bbleiffus
 * Es wird ein Objekt (wahrscheinlich DataCore) vom MonitorClient
 * gesendet und vom MonitorServer empfangen.
 * TCP Verbindung steht, aber bisher nur eine bertragung nicht dauernd !
 *
 * Revision 1.6  2002/06/07 11:21:37  bbleiffus
 * Formatierungsfehler behoben:
 * Klassenname und Konstruktor in Monitor haben nicht gestimmt
 * package Monitor in monitor gendert, da im Verzeichnis auch klein
 *
 * Revision 1.5  2002/06/06 15:29:47  bbleiffus
 * Ich bekomm's nicht ganz hin, da er mir einen Variablenfehler
 * bringt. Vielleicht schafft es einer von Euch !
 *
 * Revision 1.4  2002/06/04 21:08:47  bbleiffus
 * in das richtige Paket integriert
 * nmlich monitor nicht player
 *
 * Revision 1.3  2002/06/04 20:51:24  bbleiffus
 * im falschen package gewesen
 *
 * Revision 1.2  2002/06/04 18:19:06  bbleiffus
 * knappe Kommentare eingefgt
 *
 * Revision 1.1  2002/06/04 17:56:20  bbleiffus
 * Erste Version von Client und Server
 * NOCH NICHT GETESTET !!!
 *
 *
 */